## MTNetflixHistory
## Based on NetflixQueue -> Copyright Benjamin Trott 2002. Released under the Artistic License.
## Mods 9/22/03 by Oscar F Hills <oscar@ofhills.com> Updated to part of Netflix Suite 9_24_03
## Updated 6/28/04 adding rating tags
## Updated 8/21/05 for new Netflix site format

=head1 NAME

MTNetflixHistory - Display and format your Netflix 3-Month Rental History

=head1 USAGE

=over 4

=item 1. Drop netflix-history.pl into your plugins directory.

=item 2. Get your Netflix session ID from your browser cookies.

Log in to Netflix through your browser.

Look for the value of the NetflixShopperId cookie in your
browser preferences, after logging in. You'll use that below
where it says <your session id>

=item 3. Use the <MTNetflixHistory> tags to display your queue.

For example:

    <MTNetflixHistory session_id="<your session ID>">
    <a href="<$MTNetflixHistoryURL$>"><$MTNetflixHistoryTitle$></a><br />
    </MTNetflixHistory>

Available tags are:

    <MTNetflixHistory>
    <$MTNetflixHistoryTitle$>
    <$MTNetflixHistoryURL$>
    <$MTNetflixHistoryImage$>
    <$MTNetflixHistoryShipDate$>
    <$MTNetflixHistoryReturnDate$>
    <$MTNetflixHistoryRating$>
    <$MTNetflixHistoryRatingGraphic$>
    
    An additional argument for <MTNetfilxHistory> is "lastn"
    This treats your "history" queue as such, i.e., FIFO, but alludes to weblogs where
    the "lastn" are the "n" most recent posts. In this plugin , "lastn" returns the
    list of "n" movies at the top of your "history" queue.
    So, <MTNetflixHistory session_id = "<your session ID>" lastn = "3"> will 
    return the last 3 movies you have had out.
    
    Caution: Use of the image tag and/or ratinggraphic uses THEIR bandwith.
    
=cut

package MT::Plugin::NetflixHistory;
use strict;
use MT::Template::Context;
use LWP::UserAgent;
use HTTP::Cookies;

MT::Template::Context->add_container_tag('NetflixHistory', \&queue);
MT::Template::Context->add_tag('NetflixHistoryURL', \&queue_info);
MT::Template::Context->add_tag('NetflixHistoryImage', \&queue_info);
MT::Template::Context->add_tag('NetflixHistoryTitle', \&queue_info);
MT::Template::Context->add_tag('NetflixHistoryShipDate', \&queue_info);
MT::Template::Context->add_tag('NetflixHistoryReturnDate', \&queue_info);
MT::Template::Context->add_tag('NetflixHistoryRating', \&queue_info);
MT::Template::Context->add_tag('NetflixHistoryRatingGraphic', \&queue_info);


my $queue = 'http://www.netflix.com/RentalActivity';


sub queue {
    my($ctx, $args) = @_;
    my $session_id = $args->{session_id};
    my $lastn = defined($args->{lastn})?$args->{lastn}:0;
    my $noparms = 0;
    if (!$lastn) {
        $noparms = 1;
      }

    my $jar = HTTP::Cookies->new;
    $jar->set_cookie(0, 'NetflixShopperId', $session_id, '/', '.netflix.com');
    my $req = HTTP::Request->new(GET => $queue);
    $jar->add_cookie_header($req);
    my $ua = LWP::UserAgent->new;
    $ua->agent('NetflixWebServices/1.0');
    my $res = $ua->request($req);
    return $ctx->error("HTTP error: " . $res->status_line)
        unless $res->is_success;

    my $builder = $ctx->stash('builder');
    my $tokens = $ctx->stash('tokens');
    
    
    my $c = $res->content;
    my $out = <<EOD;
    
       <script language="javascript"><!--
var STARBAR_IMG_ROOT = 'http://cdn.netflix.com/NetFlix_Assets/pages/widget/';
var STARBAR_SET_PAGE = 'http://www.netflix.com/SetRating?foo=bar&foo=bar'
var STARBAR_MLY_PAGE = 'http://www.netflix.com/Popup?id=2010'
// --></script>
<script src='http://www.netflix.com/layout/jscript/starbar_nn.js'></script>

EOD
    
    
    $c =~ m!Returned</div>(.*?)</body>!sg;
    $c = $1;
   
 while ($c =~ m!"buf"(.*?)"brdr"></td>!sg) {
        my $rec = $1;
        my @rec = $rec =~ m!eid=(.*?)&trk!sg;
        my $mid = $1;
        $mid =~ s!&lnkctr=RecListStar!!sg;
        my $url = "http://www.netflix.com/MovieDisplay?"."movieid=".$mid;
        $rec[4] = "http://a408.g.akamai.net/f/408/1284/24h/image.netflix.com/NetFlix_Assets/boxshots/small/".$mid.".jpg";
        $rec =~ m!\d">(.*?)</a>.*?ter>(.*?)</td>.*?ter>(.*?)</td>!sg;
        my $title = $1;
        $rec[2] = $2;        
        $rec[3] = $3; 
        my $f = 0;
        if ($rec[3]) {
           if ($rec[3] eq "-") {
               $c =~ m!"buf"(.*?)ght=2></td>!sg;
               $rec = $1;
               $rec =~ m!on(.*?)\. <br>!sg;
               $rec[3] = $1;
               $f = 1;
            }
        }
        if ($f) {next;}  

        if ($lastn || $noparms) {
        if ($title && $rec[2] && $rec[3] && $rec[4]) {
        
            my $mov_url = 'http://www.netflix.com/MovieDisplay?movieid='.$mid;
            my $rate_req = HTTP::Request->new(GET => $mov_url);
            my $rate_jar = HTTP::Cookies->new;
            $rate_jar->set_cookie(0, 'NetflixShopperId', $session_id, '/', '.netflix.com');
            $rate_jar->add_cookie_header($rate_req);
            my $rate_res = $ua->request($rate_req);
            return $ctx->error("HTTP error: " . $rate_res->status_line)
              unless $res->is_success;
    
              my $rate_c = $rate_res->content;

              $rate_c =~ m!<img src=\"http\://cdn.nflximg.com/us/pages/widget/(.*?)\"!;
              $rate_c = $1;
              my $numStars = $rate_res->content;
              $numStars =~ m!class=star alt=\"(.*?) St!;
              $numStars = $1;
              $rate_c = "<img src=\"http\://cdn.nflximg.com/us/pages/widget/".$rate_c."\">";
             
              


        
            my $entry = { title => $title, url => $url, shipdate => $rec[2],
                          returndate => $rec[3], image => $rec[4], ratinggraphic => $rate_c,
                          rating => $numStars};
            $ctx->stash('netflix_history_entry', $entry);
            defined(my $o = $builder->build($ctx, $tokens))
                or return $ctx->error($builder->errstr);
            $out .= $o;
            if ($lastn){--$lastn;}
        } 
       }
    }
    $out;
}


sub queue_info {
    my $n = $_[0]->stash('netflix_history_entry');
    (my $tag = lc $_[0]->stash('tag')) =~ s!^netflixhistory!!;
    $n->{$tag};
}

1;








